// Copyright (C) 2010 Matthew Turnbull. All rights reserved.
// Use of this source code is governed by the 3-clause BSD License
// that can be found in the LICENSE file.

#ifndef __CHAT_SRV_H
#define __CHAT_SRV_H

#include <netinet/in.h>
#include <sys/socket.h>

#define MAX_MESSAGE_LENGTH 1024

#define CONNECT_FATAL -2
#define CONNECT_SOCK -1
#define CONNECT_REFUSE 0
#define CONNECT_OK 1

struct srv_sock_node {
	int sock;
	socklen_t addr_len;
	int addr_fam;
	struct sockaddr addr;
	struct srv_sock_node* next;
} typedef srv_sock_node_t;

struct client_sock_node {
	int sock;
	int error;
	socklen_t addr_len;
	int addr_fam;
	int start;
	int index;
	int length;
	char buff[MAX_MESSAGE_LENGTH];
	struct sockaddr addr;
	struct client_sock_node* next;
} typedef client_sock_node_t;

struct mesg_node {
	char* buff;
	client_sock_node_t* origin;
	struct mesg_node* next;
} typedef mesg_node_t;

int read_line(client_sock_node_t* node, char** buff);
int send_line(client_sock_node_t* node, char* buff);

const char* mynet_ntop(int af, const void *src, char *dst, int size);
int mynet_port_ntop(int af, const void *src);

client_sock_node_t* new_client_sock_node(void);
void free_client_sock_node(client_sock_node_t* node);
void free_client_sock_list(client_sock_node_t* head);

srv_sock_node_t* new_serv_sock_node(void);
void free_serv_sock_node(srv_sock_node_t* node);
void free_serv_sock_list(srv_sock_node_t* head);

int message_enqueue(char* buff, client_sock_node_t* origin);
mesg_node_t* message_dequeue(void);
void* message_dispatch_thread(void* args);
void* message_listen_thread(void* args);

int accept_connection(srv_sock_node_t *conn);
srv_sock_node_t* connect_srv_socks(char* port);

#endif

